package com.chelvc.framework.aliyun;

import java.util.Map;
import java.util.Objects;

import com.aliyun.captcha20230305.models.VerifyIntelligentCaptchaRequest;
import com.aliyun.captcha20230305.models.VerifyIntelligentCaptchaResponse;
import com.aliyun.dypnsapi20170525.models.GetMobileRequest;
import com.aliyun.dypnsapi20170525.models.GetMobileResponse;
import com.aliyun.green20220302.Client;
import com.aliyun.green20220302.models.ImageModerationRequest;
import com.aliyun.green20220302.models.ImageModerationResponse;
import com.aliyun.green20220302.models.TextModerationRequest;
import com.aliyun.green20220302.models.TextModerationResponse;
import com.aliyun.teaopenapi.models.Config;
import com.aliyun.teautil.models.RuntimeOptions;
import com.chelvc.framework.aliyun.config.AliyunProperties;
import com.chelvc.framework.common.model.Risk;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.google.common.collect.ImmutableMap;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 阿里云操作接口默认实现
 *
 * @author Woody
 * @date 2024/8/12
 */
@Slf4j
public class DefaultAliyunHandler implements AliyunHandler {
    private final AliyunProperties properties;
    private final com.aliyun.green20220302.Client green;
    private final com.aliyun.captcha20230305.Client captcha;
    private final com.aliyun.dypnsapi20170525.Client dypnsapi;
    private final RuntimeOptions runtime = new RuntimeOptions();

    public DefaultAliyunHandler(@NonNull AliyunProperties properties) {
        this.properties = properties;
        this.runtime.readTimeout = 10000;
        this.runtime.connectTimeout = 10000;

        try {
            // 初始化green客户端
            Config config = this.getConfig(properties);
            config.endpoint = "green-cip.cn-shanghai.aliyuncs.com";
            config.setRegionId("cn-shanghai");
            config.setReadTimeout(6000);
            config.setConnectTimeout(3000);
            this.green = new Client(config);

            // 初始化captcha客户端
            config = this.getConfig(properties);
            config.endpoint = "captcha.cn-shanghai.aliyuncs.com";
            this.captcha = new com.aliyun.captcha20230305.Client(config);

            // 初始化dypnsapi客户端
            config = this.getConfig(properties);
            config.endpoint = "dypnsapi.aliyuncs.com";
            this.dypnsapi = new com.aliyun.dypnsapi20170525.Client(config);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取配置对象
     *
     * @param properties 配置信息
     * @return 配置对象实例
     */
    private Config getConfig(AliyunProperties properties) {
        return new Config().setAccessKeyId(properties.getAppid()).setAccessKeySecret(properties.getSecret());
    }

    @Override
    public String code2mobile(@NonNull String token) {
        RuntimeOptions runtime = new RuntimeOptions();
        GetMobileRequest request = new GetMobileRequest().setAccessToken(token);
        GetMobileResponse response;
        try {
            response = this.dypnsapi.getMobileWithOptions(request, runtime);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        if (response == null || response.body == null) {
            throw new RuntimeException("Invalid aliyun response");
        }
        if (!Objects.equals(response.body.code, "OK")) {
            throw new RuntimeException(response.body.code + ", " + response.body.message);
        }
        return response.body.getGetMobileResultDTO().getMobile();
    }

    @Override
    public Risk verifyAvatar(@NonNull String url) {
        ImageModerationRequest request = new ImageModerationRequest();
        request.setService("profilePhotoCheck");
        request.setServiceParameters(JacksonUtils.serialize(ImmutableMap.of("imageUrl", url)));
        ImageModerationResponse response;
        try {
            response = this.green.imageModerationWithOptions(request, this.runtime);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        if (response == null || response.body == null) {
            throw new RuntimeException("Invalid aliyun response");
        }
        if (response.body.code != 200) {
            log.error("Aliyun avatar green verify failed: {}, {}", response.body.code, response.body.msg);
            return Risk.builder().level(Risk.Level.NONE).build();
        }
        Risk.Level level = Risk.Level.valueOf(response.body.data.riskLevel.toUpperCase());
        if (level != Risk.Level.NONE && ObjectUtils.notEmpty(response.body.data.result)) {
            String description = response.body.data.result.get(0).getDescription();
            return Risk.builder().level(level).description(description).build();
        }
        return Risk.builder().level(level).build();
    }

    @Override
    public Risk verifyNickname(@NonNull String nickname) {
        TextModerationRequest request = new TextModerationRequest();
        request.setService("nickname_detection");
        request.setServiceParameters(JacksonUtils.serialize(ImmutableMap.of("content", nickname)));
        TextModerationResponse response;
        try {
            response = this.green.textModerationWithOptions(request, this.runtime);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        if (response == null || response.body == null) {
            throw new RuntimeException("Invalid aliyun response");
        }
        if (response.body.code != 200) {
            log.error("Aliyun nickname green verify failed: {}, {}", response.body.code, response.body.message);
            return Risk.builder().level(Risk.Level.NONE).build();
        }
        if (StringUtils.notEmpty(response.body.data.reason)) {
            Map<?, ?> reason = JacksonUtils.deserialize(response.body.data.reason, Map.class);
            Risk.Level level = Risk.Level.valueOf(String.valueOf(reason.get("riskLevel")).toUpperCase());
            if (level != Risk.Level.NONE) {
                return Risk.builder().level(level).description(response.body.data.descriptions).build();
            }
        }
        return Risk.builder().level(Risk.Level.NONE).build();
    }

    @Override
    public boolean verifyCaptcha(@NonNull String verification) {
        VerifyIntelligentCaptchaRequest request = new VerifyIntelligentCaptchaRequest();
        request.captchaVerifyParam = verification;
        request.sceneId = this.properties.getCaptcha().getScene();
        VerifyIntelligentCaptchaResponse response;
        try {
            response = this.captcha.verifyIntelligentCaptcha(request);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        if (response == null || response.body == null || response.body.result == null) {
            throw new RuntimeException("Invalid aliyun response");
        }
        if (!Boolean.TRUE.equals(response.body.success)) {
            log.error("Aliyun intelligent captcha verify failed: {}, {}", response.body.code, response.body.message);
            return false;
        }
        if (!Boolean.TRUE.equals(response.body.result.verifyResult)) {
            log.warn("Aliyun intelligent captcha verify failed: {}", response.body.result.verifyCode);
            return false;
        }
        return true;
    }
}
