package com.chelvc.framework.common.model;

import java.io.IOException;
import java.io.Serializable;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonDeserializer;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.deser.ContextualDeserializer;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.Getter;
import lombok.NonNull;

/**
 * 自定义枚举抽象类
 *
 * @author Woody
 * @date 2025/5/17
 */
@Getter
@JsonSerialize(using = Enumeration.JacksonSerializer.class)
@JsonDeserialize(using = Enumeration.JacksonDeserializer.class)
public abstract class Enumeration implements Serializable {
    /**
     * 枚举对象JSON序列化处理器
     */
    static class JacksonSerializer extends JsonSerializer<Enumeration> {
        @Override
        public void serialize(Enumeration value, JsonGenerator generator, SerializerProvider provider)
                throws IOException {
            generator.writeString(value.getCode());
        }
    }

    /**
     * 枚举对象JSON反序列化处理器
     */
    static class JacksonDeserializer extends JsonDeserializer<Enumeration> implements ContextualDeserializer {
        private final Class<? extends Enumeration> type;

        public JacksonDeserializer() {
            this(null);
        }

        public JacksonDeserializer(Class<? extends Enumeration> type) {
            this.type = type;
        }

        @Override
        public Enumeration deserialize(JsonParser parser, DeserializationContext context) throws IOException {
            return Enumeration.parse(this.type, parser.getValueAsString());
        }

        @Override
        @SuppressWarnings("unchecked")
        public JsonDeserializer<?> createContextual(DeserializationContext context, BeanProperty property) {
            return new JacksonDeserializer((Class<? extends Enumeration>) context.getContextualType().getRawClass());
        }
    }

    /**
     * 枚举对象/值列表映射表
     */
    private static final Map<Class<?>, List<Pair<String, Boolean>>> VALUES = Maps.newConcurrentMap();

    /**
     * 枚举对象/值/实例映射表
     */
    private static final Map<Class<?>, Map<String, Enumeration>> INSTANCES = Maps.newConcurrentMap();

    /**
     * 枚举编码
     */
    private final String code;

    /**
     * 是否动态
     */
    private final boolean dynamic;

    /**
     * 描述信息
     */
    private final String description;

    protected Enumeration() {
        this(null);
    }

    protected Enumeration(String description) {
        Class<?> clazz = this.getClass();
        synchronized (clazz.getName().intern()) {
            List<Pair<String, Boolean>> values = VALUES.get(clazz);
            if (ObjectUtils.isEmpty(values) && INSTANCES.containsKey(clazz)) {
                throw new IllegalStateException("Invalid enumeration instance");
            } else if (ObjectUtils.isEmpty(values)) {
                values = VALUES.computeIfAbsent(clazz, Enumeration::getValues);
            }
            Pair<String, Boolean> pair = values.remove(0);
            this.code = Objects.requireNonNull(pair.getKey());
            this.dynamic = Boolean.TRUE.equals(pair.getValue());
            if (ObjectUtils.isEmpty(values)) {
                VALUES.remove(clazz);
            }
        }

        // 设置枚举描述信息
        this.description = ObjectUtils.ifNull(description, this.code);

        // 保存枚举值/实例映射
        INSTANCES.computeIfAbsent(clazz, k -> Maps.newConcurrentMap()).putIfAbsent(this.code, this);
    }

    /**
     * 获取枚举值
     *
     * @param clazz 枚举对象
     * @return 枚举值/是否动态信息列表
     */
    private static List<Pair<String, Boolean>> getValues(Class<?> clazz) {
        Field[] fields = clazz.getDeclaredFields();
        List<Pair<String, Boolean>> values = Lists.newLinkedList();
        if (ObjectUtils.notEmpty(fields)) {
            for (Field field : fields) {
                Class<?> type = field.getType();
                if (type.isAssignableFrom(clazz) || clazz.isAssignableFrom(type)) {
                    int modifiers = field.getModifiers();
                    if (!(type == clazz && Modifier.isPublic(modifiers) && Modifier.isStatic(modifiers)
                            && Modifier.isFinal(modifiers))) {
                        throw new IllegalStateException("Invalid enumeration field: " + field);
                    }
                    values.add(Pair.of(field.getName(), false));
                }
            }
        }
        return values;
    }

    /**
     * 枚举转换
     *
     * @param type  枚举类型
     * @param value 枚举值
     * @param <T>   对象类型
     * @return 枚举实例
     */
    @SuppressWarnings("unchecked")
    public static <T extends Enumeration> T parse(@NonNull Class<T> type, String value) {
        if (StringUtils.isEmpty(value)) {
            return null;
        }

        Enumeration instance;
        Map<String, Enumeration> instances = INSTANCES.get(type);
        if (instances == null || (instance = instances.get(value)) == null) {
            boolean dynamic;
            synchronized (type.getName().intern()) {
                if ((instances = INSTANCES.get(type)) == null) {
                    // 枚举字段尚未初始化
                    List<Pair<String, Boolean>> values = VALUES.computeIfAbsent(type, Enumeration::getValues);
                    dynamic = values.stream().noneMatch(pair -> Objects.equals(pair.getKey(), value));
                    values.add(Pair.of(value, dynamic));
                } else if ((instance = instances.get(value)) == null) {
                    // 枚举字段已经初始化
                    VALUES.put(type, Lists.newArrayList(Pair.of(value, dynamic = true)));
                } else {
                    return (T) instance;
                }

                // 优先使用无参构造函数初始化枚举实例，如果函数不存在则使用枚举描述构造函数
                Constructor<T> constructor;
                try {
                    constructor = type.getDeclaredConstructor();
                } catch (NoSuchMethodException e) {
                    try {
                        constructor = type.getDeclaredConstructor(String.class);
                    } catch (NoSuchMethodException ignore) {
                        throw new RuntimeException(e);
                    }
                }
                if (!constructor.isAccessible()) {
                    constructor.setAccessible(true);
                }
                try {
                    if (constructor.getParameterCount() > 0) {
                        instance = constructor.newInstance(value);
                    } else {
                        instance = constructor.newInstance();
                    }
                } catch (ReflectiveOperationException e) {
                    throw new RuntimeException(e);
                }
            }

            // 如果不是动态枚举则重新获取，保证一致性
            if (!dynamic) {
                instance = INSTANCES.get(type).get(value);
            }
        }
        return (T) instance;
    }

    @Override
    public int hashCode() {
        return this.code.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        return obj == this || (obj != null && obj.getClass() == this.getClass()
                && Objects.equals(this.code, ((Enumeration) obj).code));
    }

    @Override
    public String toString() {
        return this.code;
    }
}
