package com.chelvc.framework.database.interceptor;

import java.io.Serializable;
import java.lang.reflect.Type;
import java.util.ArrayDeque;
import java.util.Collections;
import java.util.Deque;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import com.baomidou.mybatisplus.core.metadata.TableFieldInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfoHelper;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.chelvc.framework.common.model.Enumeration;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.annotation.Sensitive;
import com.chelvc.framework.database.handler.EnumerationTypeHandler;
import com.chelvc.framework.database.util.SQLUtils;
import com.google.common.collect.Maps;
import lombok.Getter;
import lombok.NonNull;
import org.apache.ibatis.reflection.TypeParameterResolver;
import org.apache.ibatis.type.TypeHandler;
import org.apache.ibatis.type.UnknownTypeHandler;

/**
 * 数据表工具类
 *
 * @author Woody
 * @date 2025/3/16
 */
final class Tables {
    /**
     * 表名称/对象映射表
     */
    private static final Map<String, Table> TABLE_MAPPING = Maps.newConcurrentMap();

    /**
     * 类对象/类型处理器实例映射表
     */
    private static final Map<Class<?>, TypeHandler<?>> HANDLER_MAPPING = Maps.newConcurrentMap();

    /**
     * 上下文
     */
    private static final ThreadLocal<Deque<Context>> CONTEXT = ThreadLocal.withInitial(ArrayDeque::new);

    private Tables() {
    }

    /**
     * SQL执行上下文
     */
    @Getter
    static class Context implements Serializable {
        /**
         * 数据库Schema
         */
        private final String schema;

        /**
         * 数据源标识
         */
        private final String datasource;

        public Context(@NonNull String schema, String datasource) {
            this.schema = schema;
            this.datasource = datasource;
        }
    }

    /**
     * 表信息
     */
    @Getter
    static class Table implements Serializable {
        /**
         * 空表信息
         */
        public static final Table EMPTY = new Table();

        private final TableInfo target;
        private final Map<String, Field> fields;

        private Table() {
            this.target = null;
            this.fields = null;
        }

        public Table(@NonNull TableInfo target) {
            this.target = target;
            if (ObjectUtils.isEmpty(target.getFieldList())) {
                this.fields = Collections.emptyMap();
            } else {
                this.fields = Collections.unmodifiableMap(target.getFieldList().stream().collect(Collectors.toMap(
                        field -> SQLUtils.unquote(field.getColumn()), field -> new Field(this, field)
                )));
            }
        }

        /**
         * 获取表类型
         *
         * @return 对象类型
         */
        public Class<?> getType() {
            return this.target == null ? null : this.target.getEntityType();
        }

        /**
         * 获取表字段
         *
         * @param column 字段名
         * @return 表字段信息
         */
        public Field getField(@NonNull String column) {
            return this.fields == null ? null : this.fields.get(SQLUtils.unquote(column));
        }
    }

    /**
     * 字段信息
     */
    @Getter
    static class Field implements Serializable {
        private final Table table;
        private final boolean sensitive;
        private final TableFieldInfo target;
        private final TypeHandler<?> handler;

        public Field(@NonNull Table table, @NonNull TableFieldInfo target) {
            this.table = table;
            this.target = target;
            this.handler = getTypeHandler(table.getTarget(), target);
            this.sensitive = target.getField().isAnnotationPresent(Sensitive.class);
        }

        /**
         * 获取字段类型
         *
         * @return 对象类型
         */
        public Class<?> getType() {
            return this.target.getPropertyType();
        }

        /**
         * 获取字段对象
         *
         * @return 字段对象实例
         */
        public java.lang.reflect.Field getField() {
            return this.target.getField();
        }

        /**
         * 获取字段名称
         *
         * @return 字段名称
         */
        public String getColumn() {
            return this.target.getColumn();
        }

        /**
         * 获取属性名称
         *
         * @return 属性名称
         */
        public String getProperty() {
            return this.target.getProperty();
        }
    }

    /**
     * 获取SQL执行上下文
     *
     * @return 上下文实例
     */
    public static Context getContext() {
        return Objects.requireNonNull(CONTEXT.get().peek());
    }

    /**
     * 设置SQL执行上下文
     *
     * @param context 上下文实例
     */
    public static void setContext(@NonNull Context context) {
        CONTEXT.get().push(context);
    }

    /**
     * 移除SQL执行上下文
     */
    public static void removeContext() {
        Deque<Context> deque = CONTEXT.get();
        deque.poll();
        if (deque.isEmpty()) {
            CONTEXT.remove();
        }
    }

    /**
     * 获取简单表名，去掉schema前缀
     *
     * @param name 表名
     * @return 表名
     */
    public static String getSimpleName(String name) {
        if (StringUtils.isEmpty(name)) {
            return name;
        }
        int delimiter = name.lastIndexOf('.');
        return delimiter > 0 ? name.substring(delimiter + 1) : name;
    }

    /**
     * 获取表信息
     *
     * @param name 表名称
     * @return 表信息
     */
    public static Table getTable(@NonNull String name) {
        Context context = getContext();
        String unquote = SQLUtils.unquote(name);
        String key = context.getSchema() + StringPool.DOT + unquote;
        Table table = TABLE_MAPPING.get(key);
        if (table == null) {
            table = TABLE_MAPPING.computeIfAbsent(key, k -> {
                boolean quoted = SQLUtils.isQuoted(name);
                String datasource = context.getDatasource();
                for (TableInfo info : TableInfoHelper.getTableInfos()) {
                    String n = getSimpleName(info.getTableName());
                    if ((StringUtils.isEmpty(datasource) || info.getCurrentNamespace().startsWith(datasource))
                            && (Objects.equals(n, name) || (quoted && Objects.equals(n, unquote)))) {
                        return new Table(info);
                    }
                }
                return Table.EMPTY;
            });
        }
        return table == Table.EMPTY ? null : table;
    }

    /**
     * 获取表类型
     *
     * @param table 表名
     * @return 对象类型
     */
    public static Class<?> getTableClass(@NonNull String table) {
        return ObjectUtils.ifNull(getTable(table), Table::getType);
    }

    /**
     * 获取表字段信息
     *
     * @param table  表名称
     * @param column 字段名
     * @return 表字段信息
     */
    public static Field getTableField(@NonNull String table, @NonNull String column) {
        return ObjectUtils.ifNull(getTable(table), tb -> tb.getField(column));
    }

    /**
     * 获取字段类型处理器
     *
     * @param table 表信息
     * @param field 字段信息
     * @return 类型处理器实例
     */
    @SuppressWarnings("unchecked")
    public static TypeHandler<?> getTypeHandler(@NonNull TableInfo table, @NonNull TableFieldInfo field) {
        Class<? extends TypeHandler<?>> clazz = field.getTypeHandler();
        if (clazz == null || clazz == UnknownTypeHandler.class) {
            return null;
        } else if (clazz == EnumerationTypeHandler.class) {
            Type type = TypeParameterResolver.resolveFieldType(field.getField(), table.getEntityType());
            Class<? extends Enumeration> target = (Class<? extends Enumeration>) ObjectUtils.type2class(type);
            TypeHandler<?> handler = HANDLER_MAPPING.get(target);
            if (handler == null) {
                handler = HANDLER_MAPPING.computeIfAbsent(target, k -> new EnumerationTypeHandler(target));
            }
            return handler;
        }
        TypeHandler<?> handler = HANDLER_MAPPING.get(clazz);
        return handler != null ? handler : HANDLER_MAPPING.computeIfAbsent(clazz, k -> ObjectUtils.instance(clazz));
    }
}
