package com.chelvc.framework.email;

import java.io.UnsupportedEncodingException;
import java.nio.charset.StandardCharsets;
import java.util.function.Consumer;
import java.util.function.Function;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Store;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.search.SearchTerm;

import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.email.context.EmailContextHolder;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 邮件操作接口默认实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class DefaultEmailHandler implements EmailHandler {
    private final Session session;

    public DefaultEmailHandler(@NonNull Session session) {
        this.session = session;
    }

    /**
     * 将邮件体转换成邮件消息
     *
     * @param body        邮件体对象
     * @param attachments 邮件附件数组
     * @return 邮件消息对象
     */
    private Message body2message(Body body, MimeBodyPart... attachments) {
        Message message = new MimeMessage(this.session);
        String username = this.session.getProperty("username");
        String nickname = this.session.getProperty("nickname");
        try {
            message.addHeader("X-Mailer", "Microsoft Outlook Express 6.00.2900.2869");

            // 设置发件人
            if (StringUtils.isEmpty(nickname)) {
                message.setFrom(new InternetAddress(username));
            } else {
                message.setFrom(new InternetAddress(username, nickname, StandardCharsets.UTF_8.name()));
            }

            // 设置目标账户
            message.setRecipients(Message.RecipientType.TO, InternetAddress.parse(body.getTo()));

            // 设置抄送人
            if (StringUtils.notEmpty(body.getCc())) {
                message.setRecipients(Message.RecipientType.CC, InternetAddress.parse(body.getCc()));
            }

            // 设置暗送人
            if (StringUtils.notEmpty(body.getBcc())) {
                message.setRecipients(Message.RecipientType.BCC, InternetAddress.parse(body.getBcc()));
            }

            // 设置邮件主题
            message.setSubject(body.getSubject());
            message.setContent(EmailContextHolder.mixed(body, attachments));
        } catch (MessagingException | UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }
        return message;
    }

    @Override
    public boolean send(@NonNull Body body, @NonNull MimeBodyPart... attachments) {
        Message message = this.body2message(body, attachments);
        Transport transport = null;
        try {
            (transport = this.session.getTransport()).connect();
            transport.sendMessage(message, message.getAllRecipients());
        } catch (Exception e) {
            log.error("Email message send failed: {}", e.getMessage(), e);
            return false;
        } finally {
            if (transport != null) {
                try {
                    transport.close();
                } catch (Exception e) {
                    log.warn("Email transport close failed: {}", e.getMessage(), e);
                }
            }
        }
        return true;
    }

    @Override
    public void receive(SearchTerm search, @NonNull Consumer<Message> consumer) {
        this.receive(search, message -> {
            consumer.accept(message);
            return true;
        });
    }

    @Override
    public void receive(SearchTerm search, @NonNull Function<Message, Boolean> function) {
        Store store = null;
        Folder folder = null;
        try {
            (store = this.session.getStore()).connect();
            (folder = store.getFolder("INBOX")).open(Folder.READ_WRITE);
            Message[] messages = search == null ? folder.getMessages() : folder.search(search);
            if (ObjectUtils.notEmpty(messages)) {
                for (Message message : messages) {
                    // 防止邮件协议不能完全支持search命令，故需要再次匹配筛选条件
                    if ((search == null || message.match(search)) && Boolean.FALSE.equals(function.apply(message))) {
                        break;
                    }
                }
            }
        } catch (Exception e) {
            log.error("Email message fetch failed: {}", e.getMessage(), e);
        } finally {
            if (folder != null) {
                // 参数expunge表示是否删除标记为已删除的邮件
                try {
                    folder.close(true);
                } catch (Exception e) {
                    log.warn("Email folder close failed: {}", e.getMessage(), e);
                }
            }
            if (store != null) {
                try {
                    store.close();
                } catch (Exception e) {
                    log.warn("Email store close failed: {}", e.getMessage(), e);
                }
            }
        }
    }

    @Override
    public void delete(SearchTerm search) {
        this.receive(search, message -> {
            try {
                EmailContextHolder.delete(message);
            } catch (Exception e) {
                log.error("Email message delete failed: {}", e.getMessage(), e);
                return false;
            }
            return true;
        });
    }
}
