package com.chelvc.framework.email;

import java.io.File;
import java.util.function.Consumer;
import java.util.function.Function;
import javax.mail.Message;
import javax.mail.internet.MimeBodyPart;
import javax.mail.search.SearchTerm;

import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.email.context.EmailContextHolder;

/**
 * 邮件操作接口
 *
 * @author Woody
 * @date 2024/1/30
 */
public interface EmailHandler {
    /**
     * 发送邮件
     *
     * @param body 邮件消息体对象
     * @return true/false
     */
    default boolean send(Body body) {
        return this.send(body, new MimeBodyPart[0]);
    }

    /**
     * 发送邮件
     *
     * @param body        邮件消息体对象
     * @param attachments 邮件附件数组
     * @return true/false
     */
    default boolean send(Body body, File... attachments) {
        return this.send(body, EmailContextHolder.part(attachments));
    }

    /**
     * 发送邮件
     *
     * @param body        邮件消息体对象
     * @param attachments 邮件附件数组
     * @return true/false
     */
    boolean send(Body body, MimeBodyPart... attachments);

    /**
     * 接收邮件
     *
     * @param consumer 邮件消息回调函数
     */
    default void receive(Consumer<Message> consumer) {
        this.receive((SearchTerm) null, consumer);
    }

    /**
     * 接收邮件
     *
     * @param function 邮件消息回调函数
     */
    default void receive(Function<Message, Boolean> function) {
        this.receive((SearchTerm) null, function);
    }

    /**
     * 接收邮件
     *
     * @param search   筛选条件
     * @param consumer 邮件消息回调函数
     */
    default void receive(Search search, Consumer<Message> consumer) {
        this.receive(ObjectUtils.ifNull(search, Search::term), consumer);
    }

    /**
     * 接收邮件
     *
     * @param search   筛选条件
     * @param consumer 邮件消息回调函数
     */
    void receive(SearchTerm search, Consumer<Message> consumer);

    /**
     * 接收邮件
     *
     * @param search   筛选条件
     * @param function 邮件消息回调函数
     */
    default void receive(Search search, Function<Message, Boolean> function) {
        this.receive(ObjectUtils.ifNull(search, Search::term), function);
    }

    /**
     * 接收邮件
     *
     * @param search   筛选条件
     * @param function 邮件消息回调函数
     */
    void receive(SearchTerm search, Function<Message, Boolean> function);

    /**
     * 删除邮件
     */
    default void delete() {
        this.delete((SearchTerm) null);
    }

    /**
     * 删除邮件
     *
     * @param search 筛选条件
     */
    default void delete(Search search) {
        this.delete(ObjectUtils.ifNull(search, Search::term));
    }

    /**
     * 删除邮件
     *
     * @param search 筛选条件
     */
    void delete(SearchTerm search);
}
